/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { FloorScene } from "./FloorScene";
import { SPoint, SRect } from "@saga-web/draw/lib";
import { SceneMarkItem } from "./items/SceneMarkItem";
import { SMouseEvent } from "@saga-web/base/lib";
import { HighlightItem } from "./items/HighlightItem";
import { SMathUtil } from "./utils/SMathUtil";
import { MinDis } from "./types/MinDis";
import { PointToLine } from "./types/PointToLine";
import { RectSelectItem } from "./items/RectSelectItem";
// @ts-ignore
import { combine, difference, intersect } from "polybooljs";
// @ts-ignore
import { polygon, segments, selectUnion } from "polybooljs";
import { ShadeItem } from "./items/ShadeItem";
import { Point } from "./types/Point";
import { LikeSpaceItem } from "./items/LikeSpaceItem";
import { Poly } from "./types/Poly";
import { ZoneScene } from "./ZoneScene";

/**
 * 划分业务空间
 *
 * @author 郝建龙
 */
export class DivideFloorScene extends ZoneScene {
    /** 是否开启用户标记     */
    _isMarking: boolean = false;
    get isMarking(): boolean {
        return this._isMarking;
    } // Get isMarking
    set isMarking(v: boolean) {
        if (this._isMarking === v) {
            return;
        }
        this._isMarking = v;
    } // Set isMarking
    /** 蒙版item  */
    sceneMark: SceneMarkItem | null = null;
    /** 遮罩List  */
    shadeList: ShadeItem[] = [];
    /** 划分item  */
    cutItem: ShadeItem | null = null;
    /** 类空间itemList */
    likeSpaceList: LikeSpaceItem[] = [];
    /** 高亮item  */
    highLight: HighlightItem | null = null;
    /** 是否开启切分  */
    isCutting: boolean = false;

    /** 是否开启吸附  */
    private _isAbsorbing: boolean = false;
    get isAbsorbing(): boolean {
        return this._isAbsorbing;
    } // Get isAbsorbing
    set isAbsorbing(v: boolean) {
        this._isAbsorbing = v;
    } // Set isAbsorbing

    /** 矩形选择区域  */
    rectSelectItem: RectSelectItem | null = null;
    /** 是否开启用户选择-0:关闭,1:选择空间,2:选择业务空间   */
    _isRectSelection: number = 0;
    get isRectSelection(): number {
        return this._isRectSelection;
    } // Get isRectSelection
    set isRectSelection(v: number) {
        this._isRectSelection = v;
    } // Set isRectSelection

    /**
     * 添加轮廓线
     *
     * @param   data
     */
    addSceneMark(data: SPoint[]): void {
        if (data.length) {
            let outline = data.map(
                (t: SPoint): SPoint => {
                    t.y = -t.y;
                    return t;
                }
            );
            let sceneMark = new SceneMarkItem(null, outline);
            this.addItem(sceneMark);
            this.sceneMark = sceneMark;
        }
    } // Function addSceneMark()

    /**
     * 清除蒙版
     *
     */
    clearSceneMark(): void {
        if (this.sceneMark) {
            this.grabItem = null;
            this.removeItem(this.sceneMark);
            this.sceneMark = null;
            this.isMarking = false;
            this.view && this.view.update();
        }
    } // Function clearSceneMark()

    /**
     * 清除划分区域
     *
     */
    clearCut(): void {
        if (this.cutItem) {
            this.grabItem = null;
            this.removeItem(this.cutItem);
            this.cutItem = null;
            this.isCutting = false;
            this.view && this.view.update();
        }
    } // Function clearCut()

    /**
     * 添加所有遮罩层
     *
     * @param   data    遮罩数据
     */
    addAllShade(data: SPoint[][]): void {
        if (data.length) {
            data.forEach((t): void => {
                this.addShade(t);
            });
        }
    } // Function addAllShade()

    /**
     * 添加单个遮罩
     *
     * @param   data    遮罩数据
     */
    addShade(data: SPoint[]): void {
        let outLine = data.map(
            (t: SPoint): SPoint => {
                t.y = -t.y;
                return t;
            }
        );
        let item = new ShadeItem(null, outLine);
        this.shadeList.push(item);
        this.addItem(item);
    } // Function addShade()

    /**
     * 添加所有类空间item
     *
     * @param   data    所有数据
     */
    addAllLikeSpace(data: Point[][][]): void {
        if (data.length) {
            data.forEach((t): void => {
                this.addLikeSpace(t);
            });
        }
    } // Function addAllLikeSpace()

    /**
     * 添加单个类空间item
     *
     * @param   data    单个类空间item数据
     */
    addLikeSpace(data: Point[][]): void {
        let item = new LikeSpaceItem(null, data);
        item.selectable = true;
        item.selected = true;
        this.likeSpaceList.push(item);
        this.addItem(item);
    } // Function addLikeSpace()

    /**
     * 清除所有类空间item
     *
     */
    clearLikeSpaces(): void {
        if (this.likeSpaceList.length) {
            this.likeSpaceList.forEach((t): void => {
                this.removeItem(t);
            });
            this.likeSpaceList = [];
        }
    } // Function clearLikeSpaces()

    /**
     * 获取选中的类空间
     */
    getSelectedLikeSpace(): LikeSpaceItem[] {
        let arr: LikeSpaceItem[] = [];
        this.likeSpaceList.forEach((t): void => {
            if (t.selected) {
                arr.push(t);
            }
        });
        return arr;
    } // Function getSelectedLikeSpace()

    /**
     * 鼠标按下事件
     *
     * @param   event   保存事件参数
     * @return  boolean
     */
    onMouseDown(event: SMouseEvent): boolean {
        if (event.buttons == 1) {
            if (this.isMarking) {
                // 判断是否开启吸附,并且有吸附的点
                if (
                    this.isAbsorbing &&
                    this.highLight &&
                    this.highLight.visible
                ) {
                    event.x = this.highLight.point.x;
                    event.y = this.highLight.point.y;
                }
                if (this.isPointInShade(event.x, event.y)) {
                    return true;
                }
                if (this.sceneMark) {
                    return this.sceneMark.onMouseDown(event);
                } else {
                    let point = new SPoint(event.x, event.y);
                    let sceneMark = new SceneMarkItem(null, point);
                    this.addItem(sceneMark);
                    this.sceneMark = sceneMark;
                    this.grabItem = sceneMark;
                }
            } else if (this.isCutting) {
                // 判断是否开启吸附,并且有吸附的点
                if (
                    this.isAbsorbing &&
                    this.highLight &&
                    this.highLight.visible
                ) {
                    event.x = this.highLight.point.x;
                    event.y = this.highLight.point.y;
                }
                if (
                    this.sceneMark &&
                    this.sceneMark.contains(event.x, event.y)
                ) {
                    return true;
                }
                if (this.cutItem) {
                    return this.cutItem.onMouseDown(event);
                } else {
                    let point = new SPoint(event.x, event.y);
                    let cut = new ShadeItem(null, point);
                    this.addItem(cut);
                    this.cutItem = cut;
                    this.grabItem = cut;
                }
            } else if (this.isRectSelection) {
                if (this.rectSelectItem) {
                    this.isRectSelection = 0;
                    this.removeItem(this.rectSelectItem);
                    this.rectSelectItem = null;
                } else {
                    let point = new SPoint(event.x, event.y);
                    let rect = new RectSelectItem(null, point);
                    this.addItem(rect);
                    this.rectSelectItem = rect;
                }
            } else {
                return super.onMouseDown(event);
            }
        }
        return super.onMouseDown(event);
    } // Function onMouseDown()

    /**
     * 鼠标抬起事件
     *
     * @param   event   保存事件参数
     * @return  boolean
     */
    onMouseUp(event: SMouseEvent): boolean {
        if (this.rectSelectItem) {
            let type = this.isRectSelection;
            this.isRectSelection = 0;
            if (type == 1) {
                this.groupSelectSpace();
            } else if (type == 2) {
                this.groupSelectZone();
            }
            this.removeItem(this.rectSelectItem);
            this.rectSelectItem = null;
        }
        return false;
    } // Function onMouseUp()

    /**
     *  吸附空间
     *
     *  @param  event   鼠标事件对象
     */
    onMouseMove(event: SMouseEvent): boolean {
        if (this.rectSelectItem) {
            this.rectSelectItem.onMouseMove(event);
            return false;
        }
        super.onMouseMove(event);
        if (this.isAbsorbing) {
            if (!this.highLight) {
                this.highLight = new HighlightItem(null);
                this.addItem(this.highLight);
            }
            this.highLight.visible = false;
            if (!this.absorbShade(event)) {
                this.absorbSpace(event);
            }
        }
        return false;
    } // Function onMouseMove()

    /***
     * 键盘弹起事件
     *
     * @param      event   保存事件参数
     */
    onKeyUp(event: KeyboardEvent): void {
        if (this.isMarking || this.isCutting) {
            if (this.grabItem) {
                this.grabItem.onKeyUp(event);
            }
        } else {
            super.onKeyUp(event);
        }
    } // Function onKeyUp()

    /***
     * 注册鼠标点击事件
     *
     * @param   _this   接收者
     * @param   fn      处理函数
     */
    click(_this: FloorScene, fn: Function): void {
        this.spaceList.forEach((t): void => {
            t.connect("click", _this, fn);
        });
        this.zoneList.forEach((t): void => {
            t.connect("click", _this, fn);
        });
    } // Function click()

    /**
     *  吸附空间
     *
     *  @param  event   鼠标事件对象
     *  @return boolean 是否找到吸附的对象
     */
    absorbSpace(event: SMouseEvent): boolean {
        if (!this.highLight) {
            return false;
        }
        let absorbLen = 1000;
        if (this.view) {
            absorbLen = 10 / this.view.scale;
        }
        let P = this.absorbSpacePoint(event, absorbLen);
        if (P.Point) {
            this.highLight.distance = P.MinDis;
            this.highLight.point = new SPoint(P.Point.X, -P.Point.Y);
            this.highLight.visible = true;
            return true;
        } else {
            let L = this.absorbSpaceLine(event, absorbLen);
            if (L.Line && L.Point) {
                this.highLight.distance = L.MinDis;
                this.highLight.point = L.Point;
                this.highLight.line = L.Line;
                this.highLight.visible = true;
                return true;
            }
            return false;
        }
    } // Function absorbSpace()

    /**
     *  点是否在吸附区域内
     *
     *  @param  p       要判断的点
     *  @param  minX    空间区域
     *  @param  minY    空间区域
     *  @param  maxX    空间区域
     *  @param  maxY    空间区域
     */
    static isPointInAbsorbArea(
        p: SPoint,
        minX: number,
        maxX: number,
        minY: number,
        maxY: number
    ): boolean {
        let rect = new SRect(
            minX - 1000,
            minY - 1000,
            maxX - minX + 2000,
            maxY - minY + 2000
        );
        return rect.contains(p.x, p.y);
    } // Function isPointInAbsorbArea()

    /**
     *  吸附空间点
     *
     *  @param  event       鼠标事件对象
     *  @param  absorbLen   吸附距离
     *  @return MinDis      吸附的点
     */
    absorbSpacePoint(event: SMouseEvent, absorbLen: number): MinDis {
        let minPointDis = Number.MAX_SAFE_INTEGER;
        let Point;
        this.spaceList.map((space): void => {
            if (
                DivideFloorScene.isPointInAbsorbArea(
                    new SPoint(event.x, event.y),
                    space.minX,
                    space.maxX,
                    space.minY,
                    space.maxY
                )
            ) {
                space.data.OutLine.forEach((item): void => {
                    let minDis = SMathUtil.getMinDisPoint(
                        new SPoint(event.x, event.y),
                        item
                    );
                    if (
                        minDis &&
                        minDis.MinDis < absorbLen &&
                        minDis.MinDis < minPointDis
                    ) {
                        minPointDis = minDis.MinDis;
                        Point = minDis.Point;
                    }
                });
            }
        });
        return {
            MinDis: minPointDis,
            Point: Point
        };
    } // Function absorbSpacePoint()

    /**
     *  吸附空间线
     *
     *  @param  event       鼠标事件对象
     *  @param  absorbLen   吸附距离
     *  @return PointToLine 吸附的线
     */
    absorbSpaceLine(event: SMouseEvent, absorbLen: number): PointToLine {
        let minPointDis = Number.MAX_SAFE_INTEGER;
        let Point, Line;
        this.spaceList.forEach((space): void => {
            if (
                DivideFloorScene.isPointInAbsorbArea(
                    new SPoint(event.x, event.y),
                    space.minX,
                    space.maxX,
                    space.minY,
                    space.maxY
                )
            ) {
                space.data.OutLine.forEach((item): void => {
                    let minDisLine = SMathUtil.getMinDisLine(
                        new SPoint(event.x, event.y),
                        item
                    );
                    if (
                        minDisLine &&
                        minDisLine.MinDis < absorbLen &&
                        minDisLine.MinDis < minPointDis
                    ) {
                        minPointDis = minDisLine.MinDis;
                        Point = minDisLine.Point;
                        Line = minDisLine.Line;
                    }
                });
            }
        });
        return {
            MinDis: minPointDis,
            Point: Point,
            Line: Line
        };
    } // Function absorbSpaceLine()

    /**
     *  吸附遮罩
     *
     *  @param  event   鼠标事件对象
     *  @return boolean 是否找到吸附的对象
     */
    absorbShade(event: SMouseEvent): boolean {
        if (!this.highLight) {
            return false;
        }
        let absorbLen = 1000;
        if (this.view) {
            absorbLen = 10 / this.view.scale;
        }
        let P = this.absorbShadePoint(event, absorbLen);
        if (P.Point) {
            this.highLight.distance = P.MinDis;
            this.highLight.point = new SPoint(P.Point.X, -P.Point.Y);
            this.highLight.visible = true;
            return true;
        } else {
            let L = this.absorbShadeLine(event, absorbLen);
            if (L.Line && L.Point) {
                this.highLight.distance = L.MinDis;
                this.highLight.point = L.Point;
                this.highLight.line = L.Line;
                this.highLight.visible = true;
                return true;
            }
            return false;
        }
    } // Function absorbSpace()

    /**
     *  吸附遮罩点
     *
     *  @param  event       鼠标事件对象
     *  @param  absorbLen   吸附距离
     *  @return MinDis      吸附的点
     */
    absorbShadePoint(event: SMouseEvent, absorbLen: number): MinDis {
        let minPointDis = Number.MAX_SAFE_INTEGER;
        let Point;
        this.shadeList.forEach((shade): void => {
            let PointArr = shade.outLine.map(
                (item): Point => {
                    return { X: item.x, Y: -item.y, Z: 0 };
                }
            );
            let minDis = SMathUtil.getMinDisPoint(
                new SPoint(event.x, event.y),
                PointArr
            );
            if (
                minDis &&
                minDis.MinDis < absorbLen &&
                minDis.MinDis < minPointDis
            ) {
                minPointDis = minDis.MinDis;
                Point = minDis.Point;
            }
        });
        return {
            MinDis: minPointDis,
            Point: Point
        };
    } // Function absorbShadePoint()

    /**
     *  吸附遮罩线
     *
     *  @param  event       鼠标事件对象
     *  @param  absorbLen   吸附距离
     *  @return PointToLine 吸附的线
     */
    absorbShadeLine(event: SMouseEvent, absorbLen: number): PointToLine {
        let minPointDis = Number.MAX_SAFE_INTEGER;
        let Point, Line;
        this.shadeList.map((shade): void => {
            let PointArr = shade.outLine.map(
                (item): Point => {
                    return { X: item.x, Y: -item.y, Z: 0 };
                }
            );
            let minDisLine = SMathUtil.getMinDisLine(
                new SPoint(event.x, event.y),
                PointArr
            );
            if (
                minDisLine &&
                minDisLine.MinDis < absorbLen &&
                minDisLine.MinDis < minPointDis
            ) {
                minPointDis = minDisLine.MinDis;
                Point = minDisLine.Point;
                Line = minDisLine.Line;
            }
        });
        return {
            MinDis: minPointDis,
            Point: Point,
            Line: Line
        };
    } // Function absorbSpaceLine()

    /**
     * 计算绘制区域与选中空间交集
     *
     */
    getIntersect(): SPoint[][] {
        let arr: SPoint[][] = [];
        let poly2 = {
            regions: [],
            inverted: false
        };
        if (this.cutItem) {
            poly2.regions.push(
                // @ts-ignore
                SMathUtil.transferToArray(this.cutItem.outLine)
            );
        } else if (this.sceneMark) {
            poly2.regions.push(
                // @ts-ignore
                SMathUtil.transferToArray(this.sceneMark.outLine)
            );
        }
        if (poly2.regions.length) {
            this.spaceList.forEach((t): void => {
                if (t.selected) {
                    let poly1 = {
                        regions: [],
                        inverted: false
                    };
                    // @ts-ignore
                    poly1.regions = t.pointArr.map((item): number[][] => {
                        return SMathUtil.transferToArray(item);
                    });
                    let intersectObj = intersect(poly1, poly2);
                    if (intersectObj.regions.length) {
                        intersectObj.regions.map((t: number[][]): void => {
                            arr.push(SMathUtil.transferToSPoint(t));
                        });
                    }
                }
            });
        }
        return arr;
    } // Function getIntersect()

    /**
     * 计算绘制区域与选中类空间交集
     *
     */
    getLikeIntersect(): SPoint[][] {
        let arr: SPoint[][] = [];
        let poly2 = {
            regions: [],
            inverted: false
        };
        if (this.cutItem) {
            poly2.regions.push(
                // @ts-ignore
                SMathUtil.transferToArray(this.cutItem.outLine)
            );
        } else if (this.sceneMark) {
            poly2.regions.push(
                // @ts-ignore
                SMathUtil.transferToArray(this.sceneMark.outLine)
            );
        }
        if (poly2.regions.length) {
            this.likeSpaceList.forEach((t): void => {
                if (t.selected) {
                    let poly1 = {
                        regions: [],
                        inverted: false
                    };
                    // @ts-ignore
                    poly1.regions = t.pointArr.map((item): number[][] => {
                        return SMathUtil.transferToArray(item);
                    });
                    let intersectObj = intersect(poly1, poly2);
                    if (intersectObj.regions.length) {
                        intersectObj.regions.map((t: number[][]): void => {
                            arr.push(SMathUtil.transferToSPoint(t));
                        });
                    }
                }
            });
        }
        return arr;
    } // Function getIntersect()

    /**
     * 框选区域,选中与之相交的空间
     *
     */
    private groupSelectSpace(): void {
        if (this.rectSelectItem) {
            let item = this.rectSelectItem;
            let poly2 = {
                regions: [
                    [
                        [item.startPoint.x, item.startPoint.y],
                        [item.startPoint.x, item.endPoint.y],
                        [item.endPoint.x, item.endPoint.y],
                        [item.endPoint.x, item.startPoint.y]
                    ]
                ],
                inverted: false
            };
            this.spaceList.map(t => {
                let poly1 = {
                    regions: [],
                    inverted: false
                };
                // @ts-ignore
                poly1.regions = t.pointArr.map((item): number[][] => {
                    return SMathUtil.transferToArray(item);
                });
                let intersectObj = intersect(poly1, poly2);
                t.selected = !!intersectObj.regions.length;
            });
        }
    } // Function groupSelectSpace()

    /**
     * 判断点是否在遮罩区域内
     *
     * @param   x   点x坐标
     * @param   y   点y坐标
     * @return  boolean
     */
    isPointInShade(x: number, y: number): boolean {
        if (this.shadeList.length) {
            for (let i = 0; i < this.shadeList.length; i++) {
                if (this.shadeList[i].contains(x, y)) {
                    return true;
                }
            }
        }
        return false;
    } // Function  isPointInShade()

    /**
     * 选中的空间与区域的交集
     *
     */
    getIntersectInSpace() {
        let Obj = {};
        let poly2 = {
            regions: [],
            inverted: false
        };
        if (this.cutItem) {
            poly2.regions.push(
                // @ts-ignore
                SMathUtil.transferToArray(this.cutItem.outLine)
            );
        } else if (this.sceneMark) {
            poly2.regions.push(
                // @ts-ignore
                SMathUtil.transferToArray(this.sceneMark.outLine)
            );
        }
        if (poly2.regions.length) {
            this.spaceList.map(t => {
                let key = t.data.SourceId;
                if (t.selected) {
                    let poly1 = {
                        regions: [],
                        inverted: false
                    };
                    // @ts-ignore
                    poly1.regions = t.pointArr.map((item): number[][] => {
                        return SMathUtil.transferToArray(item);
                    });
                    let intersectObj = intersect(poly1, poly2);
                    if (intersectObj.regions.length) {
                        let outlineList = SMathUtil.getIntersectInArray(
                            intersectObj.regions
                        );
                        // @ts-ignore
                        Obj[key] = outlineList.map(t => {
                            let arr = [t.Outer];
                            t.Inner.forEach((inner): void => {
                                arr.push(inner);
                            });
                            return arr;
                        });
                    } else {
                        // @ts-ignore
                        Obj[key] = [];
                    }
                }
            });
        }
        return Obj;
    } // Function getIntersectInSpace()

    /**
     * 框选区域,选中与之相交的业务空间
     */
    private groupSelectZone(): void {
        if (this.rectSelectItem) {
            let item = this.rectSelectItem;
            const poly2: Poly = {
                regions: [
                    [
                        [item.startPoint.x, item.startPoint.y],
                        [item.startPoint.x, item.endPoint.y],
                        [item.endPoint.x, item.endPoint.y],
                        [item.endPoint.x, item.startPoint.y]
                    ]
                ],
                inverted: false
            };
            this.zoneList.forEach((t): void => {
                const poly1: Poly = {
                    regions: [],
                    inverted: false
                };
                for (let i = 0; i < t.pointArr.length; i++) {
                    poly1.regions = t.pointArr[i].map((item): number[][] => {
                        return SMathUtil.transferToArray(item);
                    });
                    let intersectObj = intersect(poly1, poly2);
                    t.selected = !!intersectObj.regions.length;
                    break;
                }
            });
        }
    } // Function groupSelectZone()

    /**
     * 选中的空间与所有业务空间的差集
     *
     * @param   flag    是否进行下一步交集预算,返回数据格式不同
     */
    getZoneDifference(flag: boolean = false) {
        const obj = {};
        let poly1: Poly = {
            regions: [],
            inverted: false
        };
        const poly2: Poly = {
            regions: [],
            inverted: false
        };
        let list: Poly[] = [],
            rect1: SRect;
        this.zoneList.forEach((t): void => {
            rect1 = t.boundingRect();
            t.pointArr.forEach((item): void => {
                let polygons: Poly = {
                    regions: [],
                    inverted: false
                };
                item.forEach((po): void => {
                    let point = SMathUtil.transferToArray(po);
                    polygons.regions.push(point);
                });
                this.spaceList.forEach((space): void => {
                    if (space.selected) {
                        let rect2 = space.boundingRect();
                        if (SMathUtil.rectIntersection(rect1, rect2)) {
                            list.push(polygons);
                        }
                    }
                });
            });
        });
        if (list.length) {
            let seg1 = segments(list[0]),
                seg2,
                comb;
            for (let i = 1; i < list.length; i++) {
                seg2 = segments(list[i]);
                comb = combine(seg1, seg2);
                seg1 = selectUnion(comb);
            }
            poly1 = polygon(seg1);
        }
        console.log(poly1);
        this.spaceList.forEach((t): void => {
            if (t.selected) {
                let key: string = t.data.SourceId;
                poly2.regions = t.pointArr.map((item): number[][] => {
                    return SMathUtil.transferToArray(item);
                });
                const diffObj: Poly = difference(poly2, poly1);
                if (diffObj.regions.length) {
                    if (flag) {
                        // @ts-ignore
                        obj[key] = diffObj;
                    } else {
                        // @ts-ignore
                        obj[key] = diffObj.regions.map(
                            (po: number[][]): SPoint[] => {
                                return SMathUtil.transferToSPoint(po);
                            }
                        );
                    }
                } else {
                    // @ts-ignore
                    obj[key] = flag ? { regions: [], inverted: false } : [];
                }
            }
        });
        return obj;
    } // getZoneIntersect()

    /**
     * 获取传入的区域与划分的区域
     *
     * @param   poly    多边形or多边形集合,由空间sourceId对应
     * */
    getPloyIntersect(poly: { ([propName]: string): Poly } | Poly) {
        const poly1: Poly = {
            regions: [],
            inverted: false
        };
        if (this.cutItem) {
            poly1.regions.push(SMathUtil.transferToArray(this.cutItem.outLine));
        } else if (this.sceneMark) {
            poly1.regions.push(
                SMathUtil.transferToArray(this.sceneMark.outLine)
            );
        }
        // @ts-ignore
        if (poly.regions) {
            return polygon(segments(intersect(poly, poly1)));
        } else {
            const obj = {};
            for (let key in poly) {
                // @ts-ignore
                obj[key] = [];
                // @ts-ignore
                if (poly[key].regions && poly[key].regions.length) {
                    // @ts-ignore
                    let interObj: Poly = intersect(poly[key], poly1);
                    interObj = polygon(segments(interObj));
                    interObj.regions.forEach((t): void => {
                        let outline = SMathUtil.transferToSPoint(t);
                        // @ts-ignore
                        obj[key].push(outline);
                    });
                }
            }
            return obj;
        }
    } // Function getPloyIntersect()
} // Class DivideFloorScene
