/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { SGraphyItem } from "@saga-web/graphy/lib";
import { SPainter, SPoint } from "@saga-web/draw/lib";
import { SMouseEvent } from "@saga-web/base/lib";
import { ItemOrder } from "../types/ItemOrder";
import { Opt } from "../types/Opt";

/**
 * 矩形选择item
 *
 * @author  郝建龙
 */
export class RectSelectItem extends SGraphyItem {
    /** 起点  */
    startPoint: SPoint = new SPoint();
    /** 终点  */
    endPoint: SPoint = new SPoint();

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param point     起点数据
     */
    constructor(parent: SGraphyItem | null, point: SPoint) {
        super(parent);
        this.startPoint = point;
        this.endPoint = new SPoint(point.x, point.y);
        this.update();
        this.zOrder = ItemOrder.rectSelectOrder;
    } // Constructor

    /**
     * 鼠标移动事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseMove(event: SMouseEvent): boolean {
        this.endPoint.x = event.x;
        this.endPoint.y = event.y;
        this.update();
        return true;
    } // Function onMouseMove()

    /**
     * Item绘制操作
     *
     * @param   painter       painter对象
     */
    onDraw(painter: SPainter): void {
        painter.pen.lineWidth = painter.toPx(2);
        painter.pen.color = Opt.rectSelectOutColor;
        painter.brush.color = Opt.rectSelectInColor;
        painter.drawRect(this.startPoint, this.endPoint);
    } // Function onDraw()
} // RectSelectItem
