/*
 * ********************************************************************************************************************
 *
 *                      :*$@@%$*:                         ;:                ;;    ;;
 *                    :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                   :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                   =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                   =@*                                  %!              @ $= % %@=   =%@!      %=
 *              *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *            %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *          ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *          $@*   ;@@@%=!:                *@*
 *          =@$    ;;;!=%@@@@=!           =@!
 *           %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *            ;%@@$=$@@%*       *@@@$=%@@%;
 *               ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

import { Opt } from "../types/Opt";
import { SGraphyItem } from "@saga-web/graphy/lib";
import { SColor, SPainter, SPoint } from "@saga-web/draw/lib";
import { SMouseEvent } from "@saga-web/base/lib";
import { ItemOrder } from "../types/ItemOrder";
import { RelationPoint } from "../types/RelationPoint";

/**
 * 蒙版item
 *
 * @author  郝建龙
 */
export class RelationItem extends SGraphyItem {
    /** 是否闭合    */
    closeFlag = false;
    /** 起点  */
    readonly startPoint = new SPoint();
    /** 起始业务空间  */
    startZone: string = "";
    /** 鼠标移动点  */
    readonly lastPoint = new SPoint();
    /** 结束业务空间  */
    endZone: string = "";
    /** 点坐标 */
    outLine: SPoint[] = [];

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data
     */
    constructor(parent: SGraphyItem | null, data: SPoint); // Constructor

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      蒙版轮廓线数据
     */
    constructor(parent: SGraphyItem | null, data: RelationPoint); // Constructor

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data
     */
    constructor(parent: SGraphyItem | null, data: SPoint | RelationPoint) {
        super(parent);
        if (data instanceof SPoint) {
            this.startPoint = data;
            this.lastPoint.x = data.x;
            this.lastPoint.y = data.y;
        } else {
            this.startZone = data.SpaceIdOne;
            this.endZone = data.SpaceIdTwo;
            this.startPoint = data.LocationOne;
            this.lastPoint = data.LocationTwo;
            this.closeFlag = true;
        }
        this.outLine.push(this.startPoint, this.lastPoint);
        this.zOrder = ItemOrder.RelationOrder;
    } // Constructor

    /**
     * 鼠标按下事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseDown(event: SMouseEvent): boolean {
        if (!this.closeFlag && event.buttons == 1) {
            this.closeFlag = true;
            this.outLine.push(this.lastPoint);
            if (this.scene) {
                // @ts-ignore
                this.scene.curRelate = null;
            }
        }
        return true;
    } // Function onMouseDown()

    /**
     * 鼠标移动事件
     *
     * @param	event         事件参数
     * @return	boolean
     */
    onMouseMove(event: SMouseEvent): boolean {
        if (!this.closeFlag) {
            this.lastPoint.x = event.x;
            this.lastPoint.y = event.y;
        }
        return true;
    } // Function onMouseMove()

    /**
     * Item绘制操作
     *
     * @param   painter       painter对象
     */
    onDraw(painter: SPainter): void {
        painter.pen.color = SColor.Red;
        painter.brush.color = SColor.Red;
        painter.pen.lineWidth = painter.toPx(5);
        painter.drawLine(this.startPoint, this.lastPoint);
        painter.pen.color = SColor.Transparent;
        painter.drawCircle(
            this.startPoint.x,
            this.startPoint.y,
            painter.toPx(8)
        );
        painter.drawCircle(this.lastPoint.x, this.lastPoint.y, painter.toPx(8));
    } // Function onDraw()
} // Class RelationItem
